//Copyright 1998 President and Fellows of Harvard University
//motifstats.cpp

#include "motifstats.h"

MotifStats::MotifStats(const char* filename){
  int i,j;
  vector<string> a,b,c;
  string s;
  ace_file=filename;

  num_motifs=number_motifs(filename);

  scn_file=new string[num_motifs+1];
  string scn_base=ace_file.substr(0,ace_file.find_last_of('.'));
  scn_base+="_n";
  for(i=1;i<=num_motifs;i++){
    scn_file[i]=scn_base+int_to_str(i)+".scn";
    //cout<<i<<'\t'<<scn_file[i]<<'\n';
  }

  ifstream fin(scn_file[1].c_str());
  if(!fin){
    cerr<<"Not able to open file "<<scn_file[1]<<'\n';
    exit(0);
  }
  string cmdline;
  getline(fin,cmdline,'\n');
  getline(fin,cmdline,'\n');
  //cout<<cmdline<<'\n';
  a=split(cmdline,' ');
  xoption=false;
  orf_file="ORF_table.txt";
  for(i=0;i<a.size();i++){
    if(a[i]=="-x"){
      xoption=true;
      //cout<<"ignored genomic features\n";
    }
    if(a[i]=="-z"){
      master_file=a[i+1];
      //cout<<master_file<<'\n';
    }
    if(a[i]=="-a"){
      orf_file=a[i+1];
    }
  }
  if(xoption){
    num_all=number_lines_beg(master_file.c_str(),">");
    //cout<<num_all<<'\n';
  }
  else{
    num_all=number_lines_beg(orf_file.c_str(),"Y");
    //cout<<num_all<<'\n';
  }

  num_input=number_lines_beg(filename,"#");
  //cout<<num_input<<'\n';
  input_seq=new string[2*num_input+1];
  ifstream ace(ace_file.c_str());
  if(!ace){cerr<<"Not able to open "<<ace_file<<'\n';exit(0);}
  i=1;
  while(getline(ace,s,'\n')){
    if(s.substr(0,1)=="#"){
      a=split(s,'\t');
      b=split(a[1],' ');
      if(xoption){
	input_seq[i]=b[0];
	if(b[0].size()==0) input_seq[i]=b[1];
	//cout<<"seq"<<i<<"\t"<<input_seq[i]<<'\n';
	i++;
      }
      else{
	c=split(b[0],'_');
	for(j=0;j<c.size();j++){
	  input_seq[i]=c[j];
	  //cout<<"seq"<<i<<"\t"<<input_seq[i]<<'\n';
	  i++;
	  if(j>0) num_input++;
	}
      }
    }
  }

  group_spec=new double[num_motifs+1];
  pos_bias=new double[num_motifs+1];
  pos_bias_pos=new int[num_motifs+1];
  pb_range=600;
  pb_bin=50;
  pb_vec=new int[pb_range];
  if(xoption) {
    calc_spec1();
    calc_pb1();
  }
  else {
    calc_spec2();
    calc_pb2();
  }
  output(cout);
}

MotifStats::~MotifStats(){
  delete [] scn_file;
  delete [] group_spec;
  delete [] pos_bias;
  delete [] input_seq;
  delete [] pb_vec;
}

void MotifStats::calc_spec1(){
  //new_style
  int i,j;
  vector<string> a;
  for(i=1;i<=num_motifs;i++){
    ifstream scn(scn_file[i].c_str());
    string s;
    bool count=false;
    int num_scan=0;
    vector<string> hit;
    hit.clear();
    while(getline(scn,s,'\n')){
      if(s.substr(0,5)=="Sites") {count=true;continue;}
      if(!count) continue;
      num_scan++;
      a=split(s,'\t');
      string site=a[0];
      hit.push_back(site);
    }
    sort(hit.begin(),hit.end());
    vector<string>::iterator p=unique(hit.begin(),hit.end());
    hit.erase(p,hit.end());
    num_scan=hit.size();
    int intersect=compare_to_input(hit);
    //ProbOverlap num_input num_scan intersect num_all
    //double r=(double)(num_input*num_scan)/(double)(intersect*num_all);
    double r=prob_overlap(num_input,num_scan,intersect,num_all);
    group_spec[i]=r;
  }
}

void MotifStats::calc_spec2(){
  //old style
  int num_compare_cutoff=100;
  int i,j;
  vector<string> a;
  for(i=1;i<=num_motifs;i++){
    ifstream scn(scn_file[i].c_str());
    string s;
    bool count=false;
    int num_scan=0;
    double score=0.0,last_score=0.0;
    vector<string> hit;
    hit.clear();
    while(getline(scn,s,'\n')){
      if(s.substr(0,5)=="Sites") {count=true;continue;}
      if(!count) continue;
      if(s.size()<2) continue;
      a=split(s,'\t');
      if(a.size()>4){
	last_score=score;
	score=str_to_dbl(a[4]);
	if(num_scan>=num_compare_cutoff&&score!=last_score) break;
      }
      a=split(s,' ');
      if(a.size()>2&&a[2]=="upstream"){
	int up=str_to_int(a[0]);
	if(up>99&&up<500){
	  string site=a[4];
	  //cout<<site<<'\n';
	  hit.push_back(site);
	  num_scan++;
	}
      }
    }
    sort(hit.begin(),hit.end());
    vector<string>::iterator p=unique(hit.begin(),hit.end());
    hit.erase(p,hit.end());
    num_scan=hit.size();
    int intersect=compare_to_input(hit);
    //ProbOverlap num_input num_scan intersect num_all
    //double r=(double)(num_input*num_scan)/(double)(intersect*num_all);
    double r=prob_overlap(num_input,num_scan,intersect,num_all);
    //cerr<<i<<'\t'<<num_input<<'\t'<<num_scan<<'\t'<<intersect<<'\t'<<num_all<<'\t'<<r<<'\n';
    group_spec[i]=r;
  }
}

int MotifStats::compare_to_input(vector<string>& a){
  int intersect=0;
  for(int i=0;i<a.size();i++){
    string site=a[i];
    for(int j=1;j<=num_input;j++){
      if(site==input_seq[j]){
	intersect++;
	break;
      }
    }
  }
  return intersect;
}

void MotifStats::calc_pb1(){
  //new_style
  int i,j,k;
  vector<string> a;
  for(i=1;i<=num_motifs;i++){
    for(j=0;j<=pb_range;j++) pb_vec[j]=0;
    ifstream scn(scn_file[i].c_str());
    string s;
    bool count=false;
    int num_scan=0;
    while(getline(scn,s,'\n')){
      if(s.substr(0,5)=="Sites") {count=true;continue;}
      if(!count) continue;
      a=split(s,'\t');
      if(str_to_int(a[1])>pb_range) continue;
      num_scan++;
      pb_vec[str_to_int(a[1])]++;
    }
    int sum_hits=0;
    for(k=1;k<=pb_bin;k++) sum_hits+=pb_vec[k];
    int best_pos=1;
    int best_sum=sum_hits;
    for(int pos=2;(pos+pb_bin)<=pb_range;pos++){
      sum_hits+=pb_vec[pos+pb_bin-1];
      sum_hits-=pb_vec[pos-1];
      if(sum_hits>best_sum){
	best_sum=sum_hits;
	best_pos=pos;
      }
    }
    double r=calc_pos_bias(best_sum,num_scan,pb_bin,pb_range);
    pos_bias[i]=r;
    pos_bias_pos[i]=best_pos;
  }
}

void MotifStats::calc_pb2(){
  //old style
  int i,j,k;
  vector<string> a;
  for(i=1;i<=num_motifs;i++){
    for(j=0;j<=pb_range;j++) pb_vec[j]=0;
    ifstream scn(scn_file[i].c_str());
    string s;
    bool count=false;
    int num_scan=0;
    while(getline(scn,s,'\n')){
      if(s.substr(0,5)=="Sites") {count=true;continue;}
      if(!count) continue;
      if(s.size()<2) continue;
      a=split(s,' ');
      if(a.size()<3) continue;
      if(a[2]!="upstream") continue;
      int up=str_to_int(a[0]);
      if(up>pb_range) continue;
      pb_vec[up]++;
      num_scan++;
    }
    int sum_hits=0;
    for(k=1;k<=pb_bin;k++) sum_hits+=pb_vec[k];
    int best_pos=1;
    int best_sum=sum_hits;
    for(int pos=2;(pos+pb_bin)<=pb_range;pos++){
      sum_hits+=pb_vec[pos+pb_bin-1];
      sum_hits-=pb_vec[pos-1];
      if(sum_hits>best_sum){
	best_sum=sum_hits;
	best_pos=pos;
      }
    }
    double r=calc_pos_bias(best_sum,num_scan,pb_bin,pb_range);
    pos_bias[i]=r;
    pos_bias_pos[i]=best_pos+pb_bin/2;
  }
}

void MotifStats::output(ostream& fout){
  for(int i=1;i<=num_motifs;i++){
    string c=ace_consensus(ace_file.c_str(),i);
    double map=ace_mapscore(ace_file.c_str(),i);
    fout.setf(ios::scientific,ios::floatfield);
    fout.precision(1);
    fout<<ace_file<<'\t'<<i<<'\t'<<map<<'\t'<<group_spec[i]<<'\t'<<pos_bias[i]<<'\t'<<pos_bias_pos[i]<<'\t'<<c<<'\n';
  }
}

double calc_pos_bias(int m, int t, int w, int s){
  int i,j,k;
  double ret=0.0;
  double r=(double)w/(double)s;
  for(i=m;i<=t;i++){
    ret+=exp( lnbico(t,i) + i*log(r) + (t-i)*log(1-r) );
  }
  return ret;
}

void cMotifStats(int argc, char* argv[]){
  if(argc<2){
    cerr<<"Usage: MotifStats ace_file\n";
    cerr<<"Results format: ace_file\tmotif_number\tmap_score\tgroup_spec\tpos_bias\tpreferred_position\tconsensus\n";
    exit(0);
  }
  string ace=argv[1];
  MotifStats(ace.c_str());
}

